{% extends "admin/change_form.html" %}
{% block extrahead %}{{ block.super }}
<script type="text/javascript">django.jQuery(document).ready(function(){ {{ dynamic_choices_binder }} });</script>
{% endblock %}
